#!/usr/bin/env python3
# coding: utf-8
import os.path
from os import path
import collections
from pprint import pprint
import argparse
import sys
import re

class convertGmap (object):
	def __init__(self):
		"""
		Global var
		"""
		# key is mrna id
		self.gffContent = collections.OrderedDict()
		self.geneCoords = { 'start': 0, 'stop' : 0, 'strand' : 'NA'}
		self.mrnaid = ''
		self.newGff = []

	def main(self):

		self.getOptions()
		self.checkInputs()

		# read gmap gff file and save into dict
		self.openGmapGFF()

		# convert all mRNA coordinates and update the gene coords if needed
		self.readGffContent()

		# create gene feature now that all mrna has been reconverted
		self.createGeneFeature()

		# create new GFF file with coordinates recalculated
		self.openOutputGffHandler()
		self.writeGffOutput()

	def createGeneFeature(self):
		attributes = 'ID='+self.options.geneid+';Name='+self.options.geneid+';cds='+self.options.cds+';mapping='+self.options.mapping
		self.geneGff = [self.options.refChrom, self.options.mapping, 'gene', str(self.geneCoords['start']), str(self.geneCoords['stop']), '.', str(self.geneCoords['strand']), '.', attributes ]
		self.geneGff = [str(i) for i in self.geneGff]

	def writeGffOutput(self):
		pprint('Create New GFF file {}'.format(self.options.output))
		self.outputGffFH.write("##gff-version 3\n")
		self.outputGffFH.write('\t'.join(self.geneGff)+"\n")
		for features in self.newGff:
			features = [str(i) for i in features]
			self.outputGffFH.write('\t'.join(features)+"\n")
		self.outputGffFH.close()

	def readGffContent(self):
		for mrnaID in self.gffContent.keys():
			pprint(" Current mRNA {}".format(mrnaID))
			self.mrnaid = mrnaID
			featurecount = {'CDS' : 0, 'exon' : 0}
			
			# deal with mrna feature first
			self.newMrnaFeature_array=self.updateMrnaFeature()
			self.newGff.append( self.newMrnaFeature_array )

			# update the gene coords if needed (aka mrna start/stop lower/greater than the current gene feature)
			self.updateGeneCoords()

			# update every child features (exons/cds/utr)
			i=1 # index 0 in array corresponds to the mrna feature which we already have updated
			while i < len(self.gffContent[self.mrnaid]):
				currentFeature = self.gffContent[self.mrnaid][i]

				# get gff feature into array
				gff = self.gffContent[self.mrnaid][i].rstrip('\n').split('\t')
				featurecount[gff[2]] +=1

				# recalc start and stop coords for the mrna feature
				newStart=self.setNewCoord(position=gff[3])
				newStop=self.setNewCoord(position=gff[4])

				# modify the attributes (remmove .path1 in the id / parents, and also the mrna1 added by gmap)
				attributes = gff[8].split(';')
				newAttributes = []
				attributeDict = collections.defaultdict()
				for info in attributes:
					(key,val) = info.split('=')
					attributeDict[key] = val

				# update the ID/Parent/CDS because, gmap add the path to the id
				attributeDict['ID'] = self.mrnaid+'.'+gff[2]+str(featurecount[gff[2]])
				attributeDict['Name'] = attributeDict['ID']
				attributeDict['Parent'] = self.mrnaid
				attributesKeys = ['ID', 'Name', 'Parent', 'Target']

				for orderedKeys in attributesKeys:
					if orderedKeys not in attributeDict.keys():
						sys.stderr.write(" Key {} not in gff attributes for feature {}".format(orderedKeys, gff))
					else:
						newAttributes.append(orderedKeys+'='+attributeDict[orderedKeys])
		
				newAttributesString = ';'.join(newAttributes)
				newGffArray = [self.options.refChrom, self.options.mapping, gff[2],newStart, newStop] + gff[5:8]  + [newAttributesString]
				newGffArray = [str(i) for i in newGffArray]
				self.newGff.append(newGffArray)
				i += 1

	def updateGeneCoords(self):
		self.geneCoords['strand'] = self.newMrnaFeature_array[6]
		if self.geneCoords['start'] == 0 and self.geneCoords['stop'] == 0:
			pprint(" New mrna for new gene: initialize gene coordinates to this mrna:")
			pprint(" mrna coords: {} {} {}".format(self.newMrnaFeature_array[0],self.newMrnaFeature_array[3],self.newMrnaFeature_array[4]))
			self.geneCoords['start'] = self.newMrnaFeature_array[3]
			self.geneCoords['stop'] = self.newMrnaFeature_array[4]
			pprint(" New gene coords: {}".format(self.geneCoords))

		else:
			if self.newMrnaFeature_array[3] < self.geneCoords['start']:
				pprint(" current mrna start lower than gene start: updating gene start to {}".format(self.newMrnaFeature_array[3]))
				self.geneCoords['start'] = self.newMrnaFeature_array[3]

			if self.newMrnaFeature_array[4] > self.geneCoords['stop']:
				pprint(" current mrna stops greater than gene stop: updating gene stop to {}".format(self.newMrnaFeature_array[4]))
				self.geneCoords['stop'] = self.newMrnaFeature_array[4]

	def updateMrnaFeature(self):
		mrnaGff = self.gffContent[self.mrnaid][0].rstrip('\n').split('\t')

		# recalc start and stop coords for the mrna feature
		newStart=self.setNewCoord(position=mrnaGff[3])
		newStop=self.setNewCoord(position=mrnaGff[4])

		# modify the attributes (remmove .path1 in the id / parents, and also the mrna1 added by gmap)
		attributes = mrnaGff[8].split(';')
		newAttributes = []
		attributeDict = collections.defaultdict()
		for info in attributes:
			(key,val) = info.split('=')
			attributeDict[key] = val

		# update the ID/Parent/CDS because, gmap add the path to the id
		attributeDict['ID'] = self.mrnaid
		attributeDict['Parent'] = self.options.geneid
		attributeDict['cds'] = self.options.cds
		attributeDict['mapping'] = self.options.mapping
		attributesKeys = ['ID', 'Name', 'Parent', 'coverage', 'identity', 'matches', 'mismatches', 'indels', 'unknowns', 'cds', 'mapping']

		for orderedKeys in attributesKeys:
			if orderedKeys not in attributeDict.keys():
				sys.stderr.write(" Key {} not in gff attributes for feature {}".format(orderedKeys, mrnaGff))
			else:
				newAttributes.append(orderedKeys+'='+attributeDict[orderedKeys])
		
		newAttributesString = ';'.join(newAttributes)
		newGffArray = [self.options.refChrom, self.options.mapping, mrnaGff[2] ,newStart, newStop] + mrnaGff[5:8]  + [newAttributesString]
		return newGffArray

	def setNewCoord(self,position):
		return int(self.options.start) + int(position)

	def openGmapGFF(self):
		lastMrnaSeen=''
		with open(self.options.gff3) as gffRecords:
			for line in gffRecords.readlines():
				if not line.startswith('#'):
					featureType = self.getGffFeature(gff=line)

					if featureType == 'mRNA':
						mrnaID = self.getFeatureAttribute(gff=line, attribute='Name')
						lastMrnaSeen = mrnaID

						self.gffContent[mrnaID] = [line]

					elif featureType == 'gene':
						continue
					
					else:
						self.gffContent[lastMrnaSeen].append(line)

		pprint(" Found {} mrna for gene {} :".format(len(self.gffContent.keys()),self.options.geneid))

	def getGffFeature(self,gff):
		return gff.rstrip('\n').split('\t')[2]

	def getFeatureAttribute(self,gff,attribute):
		attributes = gff.rstrip('\n').split('\t')[8].split(';')
		dict = collections.defaultdict()

		for info in attributes:
			(key,value) = info.split('=')
			dict[key] = value
		
		if (dict[attribute]):
			return dict[attribute]
		else:
			sys.stderr.write(' ERROR parsing attributes of GFF file at line {}'.format(gff))
			sys.stderr.write(' Cannot find attribute with key {}'.format(attribute))
			return False

	def openOutputGffHandler(self):
		self.outputGffFH=open(self.options.output, 'w')

	def getOptions(self):
		parser=argparse.ArgumentParser(description='Recalculate coordinates of a GMAP gff3 (format gene -f2) on a new reference')
		parser.add_argument('-G', '--gff3', help='GMAP result in GFF3 format (-f 2 in gmap)', required=True)
		parser.add_argument('-r', '--refChrom', help='new reference chromosome of the new target genome', required=True)
		parser.add_argument('-s', '--start', help='start offset of the new target genome', required=True)
		parser.add_argument('-e', '--end', help='end offset of the new target genome', required=True)
		parser.add_argument('-g', '--geneid', help='Gene ID to anchor', required=True)
		parser.add_argument('-c', '--cds', help='CDS status. This will be added in the final GFF attributes', required=True)
		parser.add_argument('-m', '--mapping', help='Mapping status. This will be added in the final GFF attributes', required=True)
		parser.add_argument('-o', '--output', help='Ouput GFF3', required=True)
		self.options=parser.parse_args()

	def checkFile (self,file):
		if os.path.isfile(file):
			sys.stdout.write(" File %s found\n" % str(file))
		else:
			sys.stdout.write(" ERROR: Cannot find file %s \n" % str(file))
			sys.exit()

	def checkInputs(self):
		"""
		check for input files and output directory
		"""
		sys.stdout.write(' Check input files:')
		self.checkFile(file=self.options.gff3)

if __name__== "__main__":

	run=convertGmap()
	run.main()